import java.awt.geom.Point2D;

class Door extends Entity
{
	private boolean open = false;
	private String idCard;
	
	Door (Point2D.Double p, boolean open, String idCard, String spriteID)
	{
		super(p, 16, 16);
		
		Sprite sprite = new Sprite(Store.getFrames(spriteID));
		
		setSprite(sprite);
		
		if (open)
		{
			toggle();
		}
		
		this.idCard = idCard;
	}
	
	public boolean isOpen ()
	{
		return open;
	}
	
	private void open ()
	{
		if (! open)
		{
			toggle();
		}
	}
	
	private void close ()
	{
		if (open)
		{
			toggle();
		}
	}
	
	private void toggle ()
	{
		open = (! open);
		sprite.nextFrame();
	}
	
	public void open (Player player)
	{
		if (idCard != null && player.hasCard(idCard))
		{
			open();
		}
	}
	
	public boolean solid ()
	{
		return (! open);
	}
	
	public void cycle () {}
}

class VerticalDoor extends Door
{
	VerticalDoor (Point2D.Double p, boolean open, String idCard)
	{
		super(p, open, idCard, "door-vertical");
	}
}

class HorizontalDoor extends Door
{
	HorizontalDoor (Point2D.Double p, boolean open, String idCard)
	{
		super(p, open, idCard, "door-horizontal");
	}
}
