import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.Point2D;
import java.util.*;

class GameArea extends Canvas implements MouseListener, Runnable
{
	private int width;
	private int height;
	
	private int x;
	private int y;
	
	private static GameArea instance;
	private static Thread thread;
	private static Level level;
	
	private static Level loadLevel;
	private static Point2D.Double spawn;
	
	private Image readBuffer;
	private Image writeBuffer;
	
	public static Collection <Entity> entities;
	
	public static long startTime;
	
	public void mousePressed(MouseEvent e)
	{
		requestFocus();
	}

	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseReleased(MouseEvent e)  {}

	GameArea ()
	{
		setFocusable(true);
		
		instance = this;
		
		addMouseListener(this);
	}
	
	public void init ()
	{
		requestFocus();
		
		width = getWidth();
		height = getHeight();
		
		readBuffer = createImage(width, height);
		writeBuffer = createImage(width, height);
		
		level = Store.getLevel("0");
		
		addKeyListener(Player.getInstance());
	}
	
	public void start ()
	{
		if (loadLevel != null)
		{
			Player player = Player.getInstance();
			level.removeEntity(player);
			
			level = loadLevel;
			loadLevel = null;
			
			player.setPosition(spawn);
			
			level.addEntity(player);
		}
		
		thread = new Thread(this);
		
		thread.start();
	}
	
	public void run ()
	{
		long time = System.currentTimeMillis();
		
		Graphics g = readBuffer.getGraphics();
		
		g.drawImage(Store.getImage("intro"), 0, 0, null);
		
		repaint(0);
		
		while (System.currentTimeMillis() < time + 10000) {Thread.yield();}
		
		g.drawImage(Store.getImage("station"), 0, 0, null);
		
		repaint(0);
		
		while (System.currentTimeMillis() < time + 15000) {Thread.yield();}
		
		long delay = 25;
		long then = System.currentTimeMillis();
		long now;
		long wait;
		
		startTime = then;
		
		while (thread != null)
		{
			for (Entity entity: level.entities)
			{
				if (entity != null)
				{
					entity.cycle();
				}
			}
			
			Player player = Player.getInstance();
			
			x = player.leftEdge() - width / 2 + player.outer();
			y = player.topEdge() - height / 2 + player.outer();
			
			drawBuffer();
			
			now = System.currentTimeMillis();
			wait = then + delay - now;
			
			if (wait <= 0)
			{
				Thread.yield();
			}
			else
			{
				try
			    {
					Thread.sleep(wait);
			    }
			    catch (InterruptedException e) {}
			}
			
			repaint(0);
			then = System.currentTimeMillis();
		}
		
		g = readBuffer.getGraphics();
		
		g.drawImage(Store.getImage("pop"), 0, 0, null);
		
		repaint(0);
		
		//start();
	}
	
	public static void stop ()
	{
		thread = null;
	}
	
	public static void load (String level, Point2D.Double point)
	{
		loadLevel = Store.getLevel(level);
		spawn = spawn;
		stop();
	}
	
	public void paint (Graphics g)
	{
		if (readBuffer == null)
		{
			start();
		}
		else
		{
			g.drawImage(readBuffer, 0, 0, null);
		}
	}
	
	public void update (Graphics g)
	{
		paint(g);
	}
	
	public void drawBuffer ()
	{
		Graphics g = writeBuffer.getGraphics();
		
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, width, height);
		
		level.draw(g, x, y);
		
		Image tempBuffer = readBuffer;
		
		readBuffer = writeBuffer;
		
		writeBuffer = tempBuffer;
	}
	
	public static GameArea getInstance ()
	{
		return instance;
	}
	
	public static Level getLevel ()
	{
		return level;
	}
}
