import java.awt.*;
import java.awt.geom.Point2D;
import java.util.*;

public class Level
{
	private Image image;
	private Tile[][] tiles;
	private Dimension tileSize;
	
	public final LinkedList <Entity> entities;
	
	private Queue <Entity> toAdd;
	private Queue <Entity> toRemove;
	
	public Level (Tile[][] tiles, String[] objects)
	{
		this.tiles = tiles;
		
		int rows = tiles.length;
		int cols = tiles[0].length;
		
		Image firstImage = tiles[0][0].image;
		
		int width = firstImage.getWidth(null);
		int height = firstImage.getHeight(null);
		
		tileSize = new Dimension(width, height);
		
		// Do we need a BufferedImage here?
		
		image = Store.newBufferedImage(width * cols, height * rows, false);
		
		Graphics g = image.getGraphics();
		
		for (int y = 0; y < tiles.length; y++)
		{
			for (int x = 0; x < tiles[y].length; x++)
			{
				g.drawImage(tiles[y][x].image, x * width, y * height, null);
			}
		}
		
		entities = new LinkedList <Entity> ();
		
		for (int i = 0; i < objects.length; i++)
		{
			String[] params = objects[i].split(",");
			
			int x = Integer.parseInt(params[0]);
			int y = Integer.parseInt(params[1]);
			
			Point2D.Double position = new Point2D.Double(x, y);
			
			Entity entity = null;
			
			String type = params[2];
			
			if (type.equals("Player"))
			{
				entity = new Player(position);
			}
			else if (type.equals("Scientist"))
			{
				entity = new Scientist(position);
			}
			else if (type.equals("Mechanic"))
			{
				entity = new Mechanic(position);
			}
			else if (type.equals("Guard"))
			{
				entity = new Guard(position);
			}
			else if (type.equals("Stairs"))
			{
				String updown = params[3];
				String level = params[4];
				int spawnX = Integer.parseInt(params[5]);
				int spawnY = Integer.parseInt(params[6]);
				
				entity = new Stairs(position, level, spawnX, spawnY, updown);
			}
			else if (type.equals("DoorH"))
			{
				boolean open = (params[3].equals("open"));
				
				String idCard = null;
				
				if (params.length == 5)
				{
					idCard = params[4];
				}
				
				entity = new HorizontalDoor(position, open, idCard);
			}
			else if (type.equals("DoorV"))
			{
				boolean open = (params[3].equals("open"));
				
				String idCard = null;
				
				if (params.length == 5)
				{
					idCard = params[4];
				}
				
				entity = new VerticalDoor(position, open, idCard);
			}
			else if (type.equals("PCard"))
			{
				entity = new CardPickup(position, params[3]);
			}
			/*else if (type.equals("PHealth"))
			{
				entity = new HealthPickup(position);
			}
			else if (type.equals("Regulator"))
			{
				entity = new Regulator(position);
			}
			else if (type.equals("Trigger"))
			{
				entity = new Trigger(position);
			}*/
			
			entities.add(entity);
		}
		
		while (entities.contains(null))
		{
			entities.remove(null);
		}
		
		toAdd = new LinkedList <Entity> ();
		toRemove = new LinkedList <Entity> ();
	}
	
	public Tile getTile (int x, int y)
	{
		return tiles[y][x];
	}
	
	public Tile getTileAt (Point2D.Double p)
	{
		return getTile((int) (p.x / tileSize.width), (int) (p.y / tileSize.height));
	}
	
	public Image getImage ()
	{
		return image;
	}
	
	public void draw (Graphics g, int x, int y)
	{
		g.drawImage(image, -x, -y, null);
		
		updateEntityList();
		
		for (Entity entity: entities)
		{
			if (entity != null)
			{
				entity.draw(g, x, y);
			}
		}
	}
	
	public void addEntity (Entity entity)
	{
		toAdd.offer(entity);
	}
	
	public void removeEntity (Entity entity)
	{
		toRemove.offer(entity);
	}
	
	public void updateEntityList ()
	{
		Entity entity;
		
		while ((entity = toAdd.poll()) != null)
		{
			entities.add(entity);
		}
		
		while ((entity = toRemove.poll()) != null)
		{
			entities.remove(entity);
		}
	}
}
