import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.util.*;

public class Player extends ArmedHuman implements KeyListener
{
	private static Player instance;
	
	private boolean up = false;
	private boolean down = false;
	private boolean left = false;
	private boolean right = false;
	private boolean space = false;
	private boolean shift = false;
	
	private LinkedList <String> cards;
	
	public static Player getInstance ()
	{
		return instance;
	}
	
	Player (Point2D.Double p)
	{
		super(p, 11, 16, 100);
		
		setWeapons(new Weapon[] {
			new Unarmed(this, new Sprite(Store.getFrames("player"))),
			new Pistol(this, new Sprite(Store.getFrames("player-pistol"))),
			new MachineGun(this, new Sprite(Store.getFrames("player-machinegun"))),
			new Bazooka(this, new Sprite(Store.getFrames("player-bazooka"))),
		});
		
		Sprite sprite = new Sprite(Store.getFrames("player-death"));
		
		setDeathSprite(sprite);
		
		setDirection(36);
		
		instance = this;
		
		cards = new LinkedList <String> ();
	}
	
	public void damage (int damage)
	{
		super.damage(damage);
		
		System.out.println("Health: " + health);
	}
	
	public void die ()
	{
		super.die();
		
		setWeapon(0);
		
		GameArea.getInstance().stop();
	}
	
	public void pickup (Pickup pickup)
	{
		if (pickup instanceof CardPickup)
		{
			CardPickup card = (CardPickup) pickup;
			
			cards.add(card.id);
		}
	}
	
	public boolean hasCard (String id)
	{
		for (String cardID: cards)
		{
			if (id.equals(cardID))
			{
				return true;
			}
		}
		
		return false;
	}
	
	public boolean collision (Entity e)
	{
		if (e instanceof Door)
		{
			Door door = (Door) e;
			
			door.open(this);
		}
		
		return super.collision(e);
	}
	
	public void cycle ()
	{
		if (left && ! right)
		{
			turnLeft();
		}
		else if (! left && right)
		{
			turnRight();
		}
		
		if (up && ! down)
		{
			move(shift ? 6 : 3);
			
			sprite.setFrameTime(shift ? 50 : 100);
			
			sprite.start();
		}
		else if (! up && down)
		{
			move(-3);
			sprite.start();
		}
		else
		{
			sprite.stop();
		}
		
		if (space && ! shift && weapon.automaticFire)
		{
			weapon.fire();
		}
	}
	
	public void keyPressed (KeyEvent e)
	{
		int keyCode = e.getKeyCode();
		
		if (keyCode == KeyEvent.VK_LEFT)
		{
			left = true;
		}
		
		if (keyCode == KeyEvent.VK_RIGHT)
		{
			right = true;
		}
		
		if (keyCode == KeyEvent.VK_UP)
		{
			up = true;
		}
		
		if (keyCode == KeyEvent.VK_DOWN)
		{
			down = true;
		}
		
		if (keyCode == KeyEvent.VK_SPACE)
		{
			space = true;
		}
		
		if (keyCode == KeyEvent.VK_SHIFT)
		{
			shift = true;
		}
	}
	
	public void keyReleased (KeyEvent e)
	{			
		int keyCode = e.getKeyCode();
		
		if (keyCode == KeyEvent.VK_LEFT)
		{
			left = false;
		}
		
		if (keyCode == KeyEvent.VK_RIGHT)
		{
			right = false;
		}
		
		if (keyCode == KeyEvent.VK_UP)
		{
			up = false;
		}
		
		if (keyCode == KeyEvent.VK_DOWN)
		{
			down = false;
		}
		
		if (keyCode == KeyEvent.VK_SPACE)
		{
			space = false;
		}
		
		if (keyCode == KeyEvent.VK_SHIFT)
		{
			shift = false;
		}
	}
	
	public void keyTyped (KeyEvent e)
	{
		char keyChar = e.getKeyChar();
		
		if (keyChar == ' ')
		{
			if (! weapon.automaticFire && ! shift)
			{
				weapon.fire();
			}
		}
		else if (keyChar == '0')
		{
			setWeapon(0);
		}
		else if (keyChar == '1')
		{
			setWeapon(1);
		}
		else if (keyChar == '2')
		{
			setWeapon(2);
		}
		else if (keyChar == '3')
		{
			setWeapon(3);
		}
	}
}
