import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class Store
{
	public static final Map <String, AudioClip> sounds = new HashMap <String, AudioClip> ();
	public static final Map <String, Image> images = new HashMap <String, Image> ();
	public static final Map <String, Level> levels = new HashMap <String, Level> ();
	public static final Map <String, Tile> tiles = new HashMap <String, Tile> ();
	public static final Map <String, Image[]> frames = new HashMap <String, Image[]> ();
	
	public static void load ()
	{
		addFrames("player");
		addFrames("player-pistol");
		addFrames("player-machinegun");
		addFrames("player-bazooka");
		addFrames("player-death");
		
		addFrames("scientist1");
		addFrames("scientist2");
		addFrames("scientist3");
		addFrames("scientist1-death");
		addFrames("scientist2-death");
		addFrames("scientist3-death");
		
		addFrames("guard");
		addFrames("guard-pistol");
		addFrames("guard-machinegun");
		addFrames("guard-death");
		
		addFrames("engineer");
		addFrames("engineer-fix");
		addFrames("engineer-death");
		
		addFrames("explosion");
		addFrames("spin");
		
		addFrames("door-horizontal");
		addFrames("door-vertical");
		
		addFrames("regulator");
		addFrames("regulator-broken");
		
		addImage("stairs-up");
		addImage("stairs-down");
		
		addImage("intro");
		addImage("station");
		addImage("pop");
		
		addImage("rocket");
		addImage("pickup-card");
		addImage("pickup-health");
		addImage("pickup-pistol");
		addImage("pickup-machinegun");
		
		addTiles("floor", "f");
		addTiles("wall", "w");
		addTiles("furniture", "t");
		
		addAudio("explosion");
		addAudio("pistol");
		addAudio("rocket-fire");
		addAudio("machinegun0");
		addAudio("machinegun1");
		addAudio("machinegun2");
		
		addLevel("0");
		addLevel("1");
	}
	
	public static Image[] parse (Image image)
	{
		BufferedImage bimage = toBufferedImage(image);
		
		int size = bimage.getHeight(null);
		
		int count = bimage.getWidth(null) / size;
		
		Image[] array = new Image[count];
		
		for (int i = 0; i < count; i++)
		{
			array[i] = bimage.getSubimage(i * size, 0, size, size);
		}
		
		return array;
	}
	
	public static void addFrames (String name)
	{
		frames.put(name, parse(addImage(name)));
	}
	
	public static void addTiles (String name, String key)
	{
		Image[] images = parse(addImage(name));
		
		for (int i = 0; i < images.length; i++)
		{
			tiles.put(key + i, new Tile(name, images[i]));
		}
	}
	
	public static void addAudio (String name)
	{
		try
		{
			AudioClip audio = Main.getInstance().getAudioClip(Main.getInstance().getCodeBase(), "audio/" + name + ".wav");
			sounds.put(name, audio);
			audio.play();
			audio.stop();
		}
		catch (Exception e)
		{
			System.out.println("Failed to load image file " + name);
		};
	}
	
	public static Image addImage (String name)
	{
		/*Image image = Toolkit.getDefaultToolkit().getImage(name + ".gif");
		Toolkit.getDefaultToolkit().prepareImage(image, -1, -1, null);*/
		
		Image image = Main.getInstance().getImage(Main.getInstance().getCodeBase(), "images/" + name + ".gif");
		Main.getInstance().prepareImage(image, null);
		
		images.put(name, image);
		
		return image;
	}
	
	public static void addLevel (String name)
	{
		LinkedList <String[]> lines = null;
		String[] objects = null;
		
		try
		{
			URL url = new URL(Main.getInstance().getCodeBase(), "levels/" + name + ".txt");
			
			BufferedReader d = new BufferedReader(new InputStreamReader(url.openStream()));
			
			/*BufferedReader d = new BufferedReader(new FileReader(name + ".txt"));*/
			
			String line = d.readLine();
			
			/*String[]*/ objects = line.split(" ");
			
			/*LinkedList <String[]>*/ lines = new LinkedList <String[]> ();
			
			while ((line = d.readLine()) != null)
			{
				lines.add(line.split(" "));
			}
			
		}
		catch (Exception e) { System.out.println(e); }
					
			Tile[][] grid = new Tile[lines.size()][lines.get(0).length];
			
			int y = 0;
			
			for (String[] row: lines)
			{
				for (int x = 0; x < row.length; x++)
				{
					grid[y][x] = getTile(row[x]);
					
					if (grid[y][x] == null)
					{
						System.out.println("Tile " + row[x] + "does not exist");
					}
				}
				
				y++;
			}
			
			levels.put(name, new Level(grid, objects));
		/*}
		catch (Exception e) { System.out.println(e); } */
	}
	
	public static AudioClip getAudio (String name)
	{
		return sounds.get(name);
	}
	
	public static Image getImage (String name)
	{
		return images.get(name);
	}
	
	public static Level getLevel (String name)
	{
		return levels.get(name);
	}
	
	public static Tile getTile (String name)
	{
		return tiles.get(name);
	}
	
	public static Image[] getFrames (String name)
	{
		return frames.get(name);
	}
	
	// Following three methods from http://javaalmanac.com/
	
	// This method returns a buffered image with the contents of an image
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e661 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha(image);
    
        BufferedImage bimage = newBufferedImage(image.getWidth(null), image.getHeight(null), hasAlpha);
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }
    
    public static BufferedImage newBufferedImage (int w, int h, boolean transparent)
    {
    	// Create a buffered image with a format that's compatible with the screen
        BufferedImage image = null;
        
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = (transparent ? Transparency.BITMASK : Transparency.OPAQUE);
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            image = gc.createCompatibleImage(w, h, transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (image == null) {
            // Create a buffered image using the default color model
            int type = (transparent ? BufferedImage.TYPE_INT_ARGB : BufferedImage.TYPE_INT_RGB);
            
            image = new BufferedImage(w, h, type);
        }
        
        return image;
    }
    
    // This method returns true if the specified image has transparent pixels
    public static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
        }
    
        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
         PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
        }
    
        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
    }
}
